// Copyright (C) 2006-2013 Xilinx, Inc.  All rights reserved
// $Id: Classifier.h,v 1.8.206.1 2013/04/18 17:48:35 jcorbett Exp $
#ifndef Iv_Classifier_h
#define Iv_Classifier_h

#include <cstring>
#include <string>
#include <set>

#include "XCad/export/XCad_Cad.h"
#include "JDBHelpers/XCad/export/RMed/XCad_RMed.h"

#include "CadFile.h"

// Class to help analyze and categorize various features of a placed
// and routed design.
// N.B. This class was split out of class Data.
class Classifier {
public:
   typedef std::set<std::string> NameSet;
   typedef NameSet::const_iterator NameSetIt;

   typedef std::string BlockName;
   typedef std::set<BlockName> BlockNameSet;
   typedef BlockNameSet::const_iterator BlockNameIt;

   typedef std::string NetName;
   typedef std::set<NetName> NetNameSet;
   typedef NetNameSet::const_iterator NetNameSetIt;

   template <typename NamedThing>
   class Less {
   public:
      bool operator()(const NamedThing& a,
                      const NamedThing& b) const
         {
            if (a == b) return false;
            const char* aKey(a->GetUniqueHierName());
            const char* bKey(b->GetUniqueHierName());
            return strcmp(aKey, bKey) < 0;
         }
   };

   typedef std::set<XCad_CadBlock*, Less<XCad_CadBlock*> > BlockSet;
   typedef BlockSet::const_iterator BlockSetIt;

   typedef enum NetCode {
      NoClassification,
      InterRegionSignal,
      Ground,
      Vcc,
      DriverConstant,
      DedicatedRoute,
      SourceGlobalClock,
      InsideBusMacro,
      AttachedToBusMacro,
      SourceRegionalClock,
      NonUserDcm,
      NonUser1,
      NonUser2
   } NetCode;
   typedef struct CodeHeading {
      NetCode code;
      std::string heading;
   } CodeHeading;

public:
   // Check for string property LIT_NON_USER_LOGIC="DCM_PMV_FEATURE",
   // LIT_NON_USER_LOGIC="DCM_STANDBY", or block name of
   // DCM_AUTOCALIBRATION_TOP_DCM_ADV
   bool IsDcmBlock(XCad_CadBlock* block);

   // Check for property LIT_NON_USER_LOGIC.
   bool IsNonUserBlock(XCad_CadBlock* block);

   bool IsInsideBusMacro(XCad_CadNetwork* net);
   bool IsAttachedToBusMacro(XCad_CadNetwork* net);

   bool IsDrivenByRegionalClockBuffer(XCad_CadNetwork* net);
   bool IsDrivenByGlobalClock(XCad_CadNetwork* net);

   BlockSetIt BusSetBegin();
   bool BusSetItValid(const BlockSetIt& bsIt);

   void CollectBusMacros(CadFile::NetVec& nets);

   // Return a classification code for this net.
   NetCode Classify(XCad_CadNetwork* net, XCad_RMed* rmed);

   // Return a section heading for this net.
   const std::string GetHeading(NetCode reason);

   // Return a section heading for this net.
   const std::string GetHeading(XCad_CadNetwork* net, XCad_RMed* rmed);

   // Get bus macro source.
   const std::string GetBusMacroSource(const XCad_CadBlock* block);

private:
   NetNameSet busMacroInternalNets;
   NetNameSet busMacroAttachedNets;
   BlockSet busSet;

};

#endif
