// Copyright (C) 2009-2013 Xilinx, Inc.  All rights reserved
// $Id: GroupSetFamily.h,v 1.5.206.2 2013/04/18 17:40:37 jcorbett Exp $
#ifndef Iv_GroupSetFamily_h
#define Iv_GroupSetFamily_h


#include <set>
#include <vector>
#include <algorithm>
#include <iostream>

#include "Data.h"

// Manage a collection of NameSet instances.
//
// An instance of this class is used to collect the sets of groups
// that occupy each tile. The idea is to store a pointer to a unique
// set in each tile, both to keep the tile memory small and to make it
// fast to compare tiles based on the groups that occupy them.
//
// The list is sorted prior to assigning colors to groups so that the
// colors are assigned in a predictable way.
//
// This class is a wrapper for a vector of GroupSet pointers. Each
// pointer in the list appears exactly once. Each set in the list is
// distinct. The NameSetFamily owns all the GroupSet instances it
// points to. Clients are not expected to modify nameSetList or mutate
// NameSet instances.
class GroupSetFamily {
public:
   typedef std::vector<Data::GroupSet*> GroupSetList;
   typedef GroupSetList::iterator GroupSetListIt;
   typedef GroupSetList::const_iterator GroupSetListConstIt;
public:
   GroupSetFamily();
   ~GroupSetFamily();
   Data::GroupSet* FindOrMakeSet(const Data::IsolationGroupName& name,
                                 Data::GroupSet* groupSet = 0);
   Data::GroupSet* FindOrMakeUnion(Data::GroupSet* a,
                                   Data::GroupSet* b);
   void Sort();
private:
   Data::GroupSet* Find(const Data::IsolationGroupName& name,
                        const Data::GroupSet* groupSet) const;
   bool Equal(const Data::IsolationGroupName& name,
              const Data::GroupSet* groupSet,
              const Data::GroupSet* extantSet) const;
public:
   GroupSetList groupSetList;
private:
   bool ordered;
};

bool GroupSetLess(const Data::GroupSet* a, const Data::GroupSet* b);

#endif
