// Copyright (C) 2006-2011 Xilinx, Inc.  All rights reserved
// $Id: PinChecker.h,v 1.22 2011/03/04 02:05:28 jcorbett Exp $
//
// R#check_interconnect_routing, R#check_direct_routing,
// R#check_logic_tiles, R#check_package_pins, R#check_io_banks -
// Checks package pins and IO buffers for adjacency violations using
// data from NCD files.

#ifndef Iv_PinChecker_h
#define Iv_PinChecker_h

#include <map>
#include <string>
#include <vector>

#include "TUtilities/export/smart_ptr.hpp"

#include "CadFile.h"
#include "Data.h"
#include "PinGroup.h"
#include "GridMap.h"

// Class to check that pins at the die and package level
// are sufficiently separated and that I/O banks are
// not shared between isolation groups.
class PinChecker {
public:
   PinChecker(Data& data);
   void Check();
   void ReportFaults();
   int GetStatus();
   int GetErrorCount();

private:
   typedef std::map<Rf_Cell, Rf_PackagePin> CellToPackagePinMap;
   void LabelCells(CellToPackagePinMap& pkgMap);
   void LabelCompPin(CellToPackagePinMap& pkgMap,
                     XCad_CadBlock* pinBlock,
                     XCad_CadPin* topPin);
   void LabelSpecialPin(CellToPackagePinMap& pkgMap,
                        XCad_CadPin* topPin);
   void FindPinGroup(CellToPackagePinMap& pkgMap,
                     XCad_CadPin* topPin,
                     XCad_CadPin* sigPin);
   void LabelPins(CellToPackagePinMap& pkgMap);
private:
   Data& data;
   boost::shared_ptr<PinGrid> pinGrid;
};

#endif
